/*
**
**  WAY (Where Are You ?)
**
**  load.c		database load module.
**
**  Author: Roberto Bagnara (bagnara@vxcern.cern.ch),
**          CERN  DD Division OC Group RA Section.
**
**  All rights reserved to:
**
**          CERN
**          European Laboratory for Particle Physics
**          CH 1211 Geneva 23
**          SWITZERLAND
**
**  Permission is granted to any individual or institution
**  to use, copy, or redistribute this software so long as
**  it  is not  sold for  profit, provided  this copyright
**  notice is retained.
**
**  Modification History:
**
**  Version  Date    Who              Comments
**
**  1.0.00   890902  Roberto Bagnara  First working version
**
**
**  Description:
**
**  this module contains the procedures used to load the database from
**  a file or from another name server, to dump the database contents to
**  a file or to another name server.
**
*/

/*
**      Include files.
**      --------------
*/

#include "way.h"
#include <stdio.h>
#include <strings.h>
#include "info.h"
#include "parse.h"
#include "dbase.h"

/*
**      External functions.
**      -------------------
*/

#ifdef __STDC__
extern int yyparse(void);
extern void
    init_parser(int (*input)(void), void (*action)(NS_DECLARATION_PTR decl));
extern void db_associate(const char *s_name, const char *a_name, const INFO info);
extern void db_visit_a(int (*action)(TREE_PTR node));
extern void process_keyword(char *name, char *value, INFO_PTR info);
#else
extern int yyparse();
extern void init_parser();
extern void db_associate();
extern void db_visit_a();
extern void process_keyword();
#endif

/*
**      External functions that don't like too much checking.
**      -----------------------------------------------------
*/

extern void error(/* function, format, arg1, arg2, ... */);

/*
**	Called by yyparse() inserts a declaration into the database.
**	------------------------------------------------------------
**
**
**	static void dbase_load(NS_DECLARATION_PTR decl)
**
**  Parameters:	decl		a declaration built up by the parser.
**
**  Return:	none
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Inserts the information contained in the structure pointed to by
**	"decl" into the database.
**
**  History:
**
**	02 Sep 89	Written. (RB)
**
*/

#ifdef __STDC__
static void dbase_load(NS_DECLARATION_PTR decl)
#else
static void dbase_load(decl)
NS_DECLARATION_PTR decl;
#endif
{
    register SERVICE_LIST_PTR sscan;
    register INFO_LIST_PTR iscan;
    INFO address_info;
    INFO service_info;
    INFO info;

    /* Apply the defaults. */
    
    STORE_DEFAULT_INFO(&address_info);

    /* Apply all the address-wide modifications. */

    iscan = decl->address->info;
    while (iscan) {
	process_keyword(iscan->name, iscan->value, &address_info);
	iscan = iscan->next;
    }

    /* Go through the service list. */

    sscan = decl->service_list;
    while (sscan) {

	/* Now these are the default informations. */

	service_info = address_info;

	/* Apply all the service-wide modifications. */

	iscan = sscan->info;
	while (iscan) {
	    process_keyword(iscan->name, iscan->value, &service_info);
	    iscan = iscan->next;
	}

	/* Build up the additional information for the registration. */

	info = service_info;

	/* Store all that into the database. */

	db_associate(sscan->name, decl->address->name, info);

	sscan = sscan->next;
    }
}

/*
**	Functions for loading/dumping the database contents from/to files.
**	------------------------------------------------------------------
*/

#ifdef HAS_FILE_IO

/*
**	Parser input and dump output files.
**	-----------------------------------
*/

static FILE *parser_input_file;
static FILE *dump_output_file;

void out_addinfo(node, fp)
LIST_PTR node;
FILE *fp;
{
    fprintf(fp, "(version=%d, compatibility=%d, package_no=%d, timeout=%d)",
    		node->info.long_info.longs.version,
    		node->info.long_info.longs.compatibility,
    		node->info.long_info.longs.package_no,
    		node->info.long_info.longs.timeout);
}

#ifdef __STDC__
int formatted_output(TREE_PTR node)
#else
int formatted_output(node)
TREE_PTR node;
#endif
{
    LIST_PTR tmp;
    FILE *fp = dump_output_file;

    (void) fprintf(fp, "%s ->\n", node->name);
    for (tmp=node->assoc; tmp; tmp = tmp->a_next) {
	(void) fprintf(fp, "\t%s", tmp->s_node->name);
	out_addinfo(tmp, fp);
	if (tmp->a_next)
	    (void) fputs(",\n", fp);
    }
    (void) fputs("\n;\n", fp);
    return 0;
}

/*
**	File input routine for the parser.
**	----------------------------------
*/

#ifdef __STDC__
static int parser_file_input(void)
#else
static int parser_file_input()
#endif
{
    return getc(parser_input_file);
}

/*
**
**	Load the database from a file.
**	------------------------------
**
**
**	int dbase_load_from_file(char *file_name)
**
**  Parameters:	file_name	name of the input file.
**
**  Return:			1 on success, 0 on failure.
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Initialize the database contents by reading and parsing the file
**	whose name is "file_name". If "file_name" is null or "-" then stdin
**	is used as the input file. A 0 completion code is returned if the
**	specified input file does not exist or if a parsing error occurs.
**	A 1 completion code means that all was ok.
**
**  History:
**
**	02 Sep 89	Written. (RB)
**
*/

#ifdef __STDC__
int dbase_load_from_file(char *file_name)
#else
int dbase_load_from_file(file_name)
char *file_name;
#endif
{
    int parser_completion;

    /* Try to open the input file for read. */

    if (file_name == (char *) 0 || !strcmp(file_name, "-")) {
	parser_input_file = stdin;
    } else {
	parser_input_file = fopen(file_name, "r");
	if (parser_input_file == 0) {
	    error("dbase_load_from_file", "cannot open `%s'", file_name);
	    return 0;
	}
    }

    /* Initialize the parser. */

    init_parser(parser_file_input, dbase_load);

    /* Go parsing, close the input file and report the status. */

    parser_completion = yyparse() ? 0 : 1;
    (void) fclose(parser_input_file);

    return parser_completion;
}

/*
**
**	Dump the database contents to a file.
**	-------------------------------------
**
**
**	int dbase_load_from_file(char *file_name)
**
**  Parameters:	file_name	name of the output file.
**
**  Return:			1 on success, 0 on failure.
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Dumps the database contents to the file whose name is "file_name".
**	If "file_name" is null or "-" then stdout is used as the output file.
**	A 0 completion code is returned if the specified output file cannot
**	be opened. A 1 completion code means that all was ok.
**
**  History:
**
**	02 Sep 89	Written. (RB)
**
*/

#ifdef __STDC__
int dbase_dump_to_file(char *file_name)
#else
int dbase_dump_to_file(file_name)
char *file_name;
#endif
{
    /* Try to open the output file for write. */

    if (file_name == (char *) 0 || !strcmp(file_name, "-")) {
	dump_output_file = stdout;
    } else {
	dump_output_file = fopen(file_name, "w");
	if (dump_output_file == 0) {
	    error("dbase_dump_to_file", "cannot open `%s'", file_name);
	    return 0;
	}
    }

    /* Print the database contents, then close the output file. */

    db_visit_a(formatted_output);
    (void) fclose(dump_output_file);
    return 1;
}

#endif /* HAS_FILE_IO */
