/*
**
**  WAY (Where Are You ?)
**
**  select.c		address selection module.
**
**  Author: Roberto Bagnara (bagnara@vxcern.cern.ch),
**          CERN  DD Division OC Group RA Section.
**
**  All rights reserved to:
**
**          CERN
**          European Laboratory for Particle Physics
**          CH 1211 Geneva 23
**          SWITZERLAND
**
**  Permission is granted to any individual or institution
**  to use, copy, or redistribute this software so long as
**  it  is not  sold for  profit, provided  this copyright
**  notice is retained.
**
**  Modification History:
**
**  Version  Date    Who              Comments
**
**  1.0.00   890815  Roberto Bagnara  First working version
**
**
**  Description:
**
**  this module contains the procedure to select an address...
**
*/

/*
**      Include files.
**      --------------
*/

#include "way.h"
#include <strings.h>
#include "info.h"
#include "dbase.h"

/*
**      External functions.
**      -------------------
*/

#ifdef __STDC__
extern LIST_PTR db_search_s(const char *name);
extern void db_dissociate(const char *s_name, const char *a_name);
extern int glob_match(char *pattern, char *text);
extern char *xmalloc(int size);
extern void xfree();
#else
extern LIST_PTR db_search_s();
extern void db_dissociate();
extern int glob_match();
extern char *xmalloc();
extern void xfree();
#endif

/*
**      External functions that don't like too much checking.
**      -----------------------------------------------------
*/

extern void wlog(/* format, arg1, arg2, ... */);
extern void tlog(/* format, arg1, arg2, ... */);

/*
**	External variables.
**	-------------------
*/

extern int log_level;


/*
**
**	The address selection function.
**	-------------------------------
**
**
**	LIST_PTR select_best_address(const char *service, int version,
**				int compatibility, const char *pref_addresses)
**
**  Parameters:	service		name of the required service;
**		pref_addresses	comma separated list of preferred addresses.
**
**  Return:			pointer to a tree node or NULL_TREE_PTR.
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Select from the database the best address providing the functionality
**	denoted by "service" and matching a pattern in the best-first sorted
**	list of preferred addresses "pref_addresses". If the search succeeds
**	a pointer to the association list element is returned, otherwise
**	NULL_LIST_PTR is returned.
**
**				WARNING
**
**		THE ALGORITHM USED HERE FOR DETERMINING
**		THE "BEST ADDRESS" IS PROVISIONAL, AND
**		PROBABLY SUBJECT TO CHANGE MANY TIMES !!!
**
**  History:
**
**	28 Jul 89	Written. (RB)
**
*/

#define IS_AUTHORIZED(dummy) 1
#define IS_REACHABLE(dummy) 1
#define IS_ALIVE(dummy) 1

#ifdef __STDC__
LIST_PTR select_best_address(const char *service, int version,
				int compatibility, const char *pref_addresses)
#else
LIST_PTR select_best_address(service, version, compatibility, pref_addresses)
char *service;
int version;
int compatibility;
char *pref_addresses;
#endif
{
    char *pattern_sequence;
    char *pattern;
    char *next_pattern;

    LIST_PTR address_list;
    LIST_PTR tmp;

    int last = 0;

    /* See if service is known to the database module. */

    if ((address_list = db_search_s(service)) == NULL_LIST_PTR) {
	LOG(2) wlog("The database does not contain any reference to service %s .",
    	    service);
	return NULL_LIST_PTR;
    }

    /* Setup our copy of the preferred addresses patterns. */

    pattern_sequence = xmalloc(strlen(pref_addresses) + 1);
    (void) strcpy(pattern_sequence, pref_addresses);

    /* Try to find the best address who provides the required service. */

    pattern = pattern_sequence;
    do {

	/* Look for a comma, if found substitute it with '\0' to mark the
	   end of the pattern. If not found remember that this is our last
	   go.
	*/

	if (next_pattern = strchr(pattern, ',')) {
	    *next_pattern++ = '\0';
	} else
	    last = 1;

	/* Scan the address list to find an address providing a version
	   of the server acceptable to the client, for which the client
	   have authorized access, and that, to the best of our knowledge
	   is active.
	*/

	for (tmp = address_list; tmp != NULL_LIST_PTR; tmp = tmp->s_next) {

	    char *address = tmp->a_node->name;

		/* Here is the connectivity check. */

	    if (glob_match(pattern, address) && (

		/* Here is the version check. */

		(tmp->info.long_info.longs.version < version) ?
		    (tmp->info.long_info.longs.version >= compatibility) :
		    (version >= tmp->info.long_info.longs.compatibility)

		/* Here is the authorization check. */

		) && IS_AUTHORIZED(tmp))

		/* We have an address that the client could accept, let's
		   see if we can do something more for him.
		*/

		if (IS_POLL_ALLOWED(tmp->info.flags) && IS_REACHABLE(address)) {

		    /* We are allowed to poll the server, so we can check if
		       it is alive. At the same time, and for the same price,
		       we can fetch the package number on behalf of the client.
		    */

		    if (IS_ALIVE(address))

			/* Server is alive, our job is finished. */

			break;
		    else

			/* Server is dead, if the association is not static
			   we can remove it, the server will register again.
			*/

			if (!IS_STATIC(tmp->info.flags))
			    db_dissociate(service, address);

			/* Carry on searching (no break). */
		} else

		    /* We cannot check if the server is alive, the client
		       will see.
		    */

		    break;
	}

	/* Try next address pattern. */

	pattern = next_pattern;
    } while (!last);

    /* Release memory. */

    xfree(pattern_sequence);
    return tmp;
}
