/*
**
**  WAY (Where Are You ?)
**
**  util.c		utility module.
**
**  Author: Roberto Bagnara (bagnara@vxcern.cern.ch),
**          CERN  DD Division OC Group RA Section.
**
**  All rights reserved to:
**
**          CERN
**          European Laboratory for Particle Physics
**          CH 1211 Geneva 23
**          SWITZERLAND
**
**  Permission is granted to any individual or institution
**  to use, copy, or redistribute this software so long as
**  it  is not  sold for  profit, provided  this copyright
**  notice is retained.
**
**  Modification History:
**
**  Version  Date    Who              Comments
**
**  1.0.00   890815  Roberto Bagnara  First working version
**
**
**  Description:
**
**  this module contains some utility routines used by other modules of
**  the name server. The storage allocators, the fatal and normal error
**  message functions and the information logging function are provided
**  here.
*/

/*
**      Include files.
**      --------------
*/

#include "way.h"

#if defined (vms)
#include <stdlib.h>
#else
#if defined (__MSDOS__)
#include <alloc.h>
#else
#include <malloc.h>
#endif
#endif

#include <stdio.h>
#include <varargs.h>

#if defined (SYSV)
#include <time.h>
#else
#include <sys/time.h>
#endif

#if defined (SYSV)
#include <string.h>
#else
#include <strings.h>
#endif


/*
**      External functions.
**      -------------------
*/

extern void exit();

/*
**
**	Error reporting function.
**	-------------------------
**
**	
**	void error(va_alist) called as error("function", fmt, arg1, arg2, ...)
**
**  Parameters:	none
**
**  Return:	none
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Reports an error message.
**
**  History:
**
**	15 Aug 89	Written. (RB)
**
*/

void error(va_alist)
va_dcl
{
    va_list args;
    char *fmt;
    char buf[200];

    va_start(args);
    (void) sprintf(buf, "ERROR (%s): ", va_arg(args, char *));
    fmt = va_arg(args, char *);
    (void) vsprintf(buf + strlen(buf), fmt, args);
    va_end(args);

    fprintf(stderr, "%s\n", buf);
}


/*
**
**	Fatal error function.
**	---------------------
**
**	
**	void fatal(va_alist) called as fatal("function", fmt, arg1, arg2, ...)
**
**  Parameters:	none
**
**  Return:	NONE		DOES NOT RETURN TO CALLER
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Reports an error message, then calls exit().
**
**  History:
**
**	15 Aug 89	Written. (RB)
**
*/

void fatal(va_alist)
va_dcl
{
    va_list args;
    char *fmt;
    char buf[200];

    va_start(args);
    (void) sprintf(buf, "FATAL (%s): ", va_arg(args, char *));
    fmt = va_arg(args, char *);
    (void) vsprintf(buf + strlen(buf), fmt, args);
    va_end(args);

    fprintf(stderr, "%s\n", buf);
    exit(FATAL_EXIT_CODE);
}


/*
**
**	Information logging functions.
**	------------------------------
**
**	
**      void wlog(va_alist) called as wlog(fmt, arg1, arg2, ...)
**      void tlog(va_alist) called as tlog(fmt, arg1, arg2, ...)
**
**  Parameters:	none
**
**  Return:	none
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**      Logs the passed message. tlog() prints out, in addition to wlog(),
**	the date and time before the actual message.
**
**  History:
**
**	15 Aug 89	Written. (RB)
**
*/

void wlog(va_alist)
va_dcl
{
    va_list args;
    char *fmt;
    char buf[200];
    extern FILE *log_file;

    va_start(args);
    fmt = va_arg(args, char *);
    (void) vsprintf(buf, fmt, args);
    va_end(args);

    fprintf(log_file, "%s\n", buf);
    fflush(log_file);
}

void tlog(va_alist)
va_dcl
{
    va_list args;
    char *fmt;
    char buf[200];
    extern FILE *log_file;

    long t;
    static struct tm *timebuf = NULL;
    struct tm *localtime ();
    static char *monthnames[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
				 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
				};

    if (timebuf == NULL) {
	t = time(0);
	timebuf = localtime(&t);
    }
#ifndef cray
    sprintf(buf, "*** %s %2d %4d %02d:%02d:%02d ***\n",
		monthnames[timebuf->tm_mon],
		timebuf->tm_mday, timebuf->tm_year + 1900,
		timebuf->tm_hour, timebuf->tm_min, timebuf->tm_sec);
#endif
    va_start(args);
    fmt = va_arg(args, char *);
    (void) vsprintf(buf + strlen(buf), fmt, args);
    va_end(args);

    fprintf(log_file, "%s\n", buf);
    fflush(log_file);
}


/*
**
**	Storage de-allocator.
**	---------------------
**
**	
**	void xfree(char *ptr)
**
**  Parameters:	ptr		pointer to the area to deallocate.
**
**  Return:	none
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Deallocates the memory space previously allocated by xmalloc or
**	xrealloc.
**
**  History:
**
**	15 Aug 89	Written. (RB)
**
*/

#ifdef __STDC__
void xfree(char *ptr)
#else
void xfree(ptr)
char *ptr;
#endif
{
    free(ptr);
}


/*
**
**	Storage allocator.
**	------------------
**
**	
**	char *xmalloc(int size)
**
**  Parameters:	size		size in bytes of the required area.
**
**  Return:			pointer to the newly allocated area, if
**				allocation succeeds, it wont return otherwise.
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Allocates "size" bytes of memory space returning a pointer to the
**	newly allocated area. If the operation fails (i.e. no memory
**	available) fatal is called.
**
**  History:
**
**	15 Aug 89	Written. (RB)
**
*/

#ifdef __STDC__
char *xmalloc(int size)
#else
char *xmalloc(size)
int size;
#endif
{
    register char *tmp;

    if ((tmp = (char *) malloc(size)) == (char *) 0)
	fatal("xmalloc", "not enough dynamic memory, aaaaiiiieeee !!!");
    return tmp;
}


/*
**
**	Storage re-allocator.
**	------------------
**
**	
**	char *xrealloc(char *ptr, int size)
**
**  Parameters: ptr		pointer to the memory area previously
**				allocated with xmalloc;
**		size		new size in bytes of the required area.
**
**  Return:			pointer to the newly allocated area, if
**				allocation succeeds, it wont return otherwise.
**
**  On entry:	none
**
**  On exit:	none
**
**  Description:
**
**	Extends the memory area previously allocated with xmalloc to "size"
**	bytes returning a pointer to the newly allocated area. If the
**	operation fails (i.e. no memory available) fatal() is called.
**
**  History:
**
**	15 Aug 89	Written. (RB)
**
*/

#ifdef __STDC__
char *xrealloc(char *ptr, int size)
#else
char *xrealloc(ptr, size)
char *ptr;
int size;
#endif
{
    register char *tmp;

    if ((tmp = (char *) realloc(ptr, size)) == (char *) 0)
	fatal("xrealloc", "not enough dynamic memory, aaaaiiiieeee !!!");
    return tmp;
}
